package org.kar.karanage.api;

import org.kar.archidata.GlobalConfiguration;
import org.kar.archidata.SqlWrapper;
import org.kar.archidata.WhereCondition;
import org.kar.karanage.model.DataLog;
import org.kar.karanage.model.Group;
import org.kar.karanage.model.MultipleLogElement;
import jakarta.ws.rs.core.Response;

import org.kar.archidata.annotation.SQLComment;
import org.kar.archidata.annotation.SQLCreateTime;
import org.kar.archidata.annotation.SQLDefault;
import org.kar.archidata.annotation.SQLForeignKey;
import org.kar.archidata.annotation.SQLIfNotExists;
import org.kar.archidata.annotation.SQLLimitSize;
import org.kar.archidata.annotation.SQLNotNull;
import org.kar.archidata.annotation.SQLPrimaryKey;
import org.kar.archidata.annotation.SQLUpdateTime;
import org.kar.archidata.annotation.SQLTableLinkGeneric.ModelLink;
import org.kar.archidata.annotation.security.PermitAll;
import org.kar.archidata.db.DBEntry;
import org.kar.archidata.exception.FailException;
import org.kar.archidata.exception.InputException;

import jakarta.ws.rs.*;
import jakarta.ws.rs.core.MediaType;

import java.lang.reflect.Field;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;


@Path("/log")
@Produces({MediaType.APPLICATION_JSON})
public class LogResource {
 
	
    @GET
    @Path("{group}/{system}")
    //@RolesAllowed("USER")
    @PermitAll
    public static String get(
    		@PathParam("group") String groupName,
    		@PathParam("system") String system,
    		@QueryParam("since") String since,
    		@QueryParam("sinceId") Integer sinceId,
    		@QueryParam("limit") Integer limit,
    		@QueryParam("mode") String mode) throws Exception {
    	// Keep Group ID:
    	if (groupName == null || groupName.length() == 0) {
    		throw new InputException("group", "Missing URL parameter /state/{group}/...");
    	}
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /state/{group}/... ==> Unknown group name");
    	}
    	if (system == null || system.length() == 0) {
    		throw new InputException("system", "Missing URL parameter /state/xxx/{system}/...");
    	}
    	if (limit == null || limit == 0) {
    		limit = 100;
    	}
    	if (limit > 100) {
    		limit = 100;
    	}
    	if (limit < 0) {
    		throw new InputException("limit", "Limit must be inside [1..100]");
    	}
    	if (since != null && sinceId != null) {
    		throw new InputException("since,sinceId", "The 2 parameters can not be set at the same time...");
    	}
    	
    	List<DataLog> datas = null;
    	if (since != null) {
    		Timestamp time = null;
    		try {
    			time = Timestamp.from(Instant.parse(since));
    		}  catch (Exception ex) {
    			throw new InputException("since", "url: ?since=... ==> is not an iso 8601 time format");
    		}
    		datas = SqlWrapper.getsWhere(DataLog.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("system", "=", system),
    						new WhereCondition("create_date", ">", time)
    						),
    				"create_date",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "system has no new data or does not exist: '" + system + "'");
        	}
    	} else if (sinceId != null) {
    		datas = SqlWrapper.getsWhere(DataLog.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("system", "=", system),
    						new WhereCondition("id", ">", sinceId)
    						),
    				"id",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "system has no new data or does not exist: '" + system + "'");
        	}
    	} else {
    		datas = SqlWrapper.getsWhere(DataLog.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("system", "=", system)
    						),
    				"id",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "system does not exist: '" + system + "'");
        	}
    	}

    	StringBuilder out = new StringBuilder("[");
    	boolean first = true;
    	for (DataLog data : datas) {
    		if (first) {
    			first = false;
    		} else {
    			out.append(",");
    		}
    		out.append("{ \"id\": " + data.id + ", \"time\": ");

        	if ("raw".equals(mode)) {
        		double seconds = data.create_date.toInstant().toEpochMilli() * 0.001;
        		out.append(seconds);
        	} else {
        		out.append("\"");
        		out.append(data.create_date.toInstant().toString());
        		out.append("\"");
        	}
        	out.append(", \"data\":" + data.data + "}");
    	}
    	out.append("]");
    	return out.toString();
    }
	
    @POST
    @Path("{group}/{system}")
    @PermitAll
    @Consumes(MediaType.APPLICATION_JSON)
    public Response post(
    		@PathParam("group") String groupName,
    		@PathParam("system") String system, 
    		@QueryParam("time") String time,
    		@QueryParam("id") Long clientId,
    		@QueryParam("uuid") Long clientUuid,
    		String dataToInsert) throws Exception {
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /log/{group}/... ==> Unknown group name");
    	}
    	DataLog data = new DataLog();
    	data.group = group.id;
    	data.system = system;
    	data.data = dataToInsert;
    	if (clientId != null) {
    		data.clientId = clientId;
    	}
    	if (clientUuid != null) {
    		data.clientUuid = clientUuid;
    	}
    	if (time != null) {
    		try {
    			data.create_date = Timestamp.from(Instant.parse(time));
    		}  catch (Exception ex) {
    			throw new InputException("time", "url: ?time=... ==> is not an iso 8601 time format");
    		}
    	}
    	SqlWrapper.insert(data);
		return Response.status(201).build();
    }
    @POST
    @Path("{group}/{system}/push_multiple")
    @PermitAll
    @Consumes(MediaType.APPLICATION_JSON)
    public Response post(
    		@PathParam("group") String groupName,
    		@PathParam("system") String system,
    		@QueryParam("uuid") Long clientUuid,
    		List<MultipleLogElement> listDataToInsert) throws Exception {
    	long start_time = System.currentTimeMillis();
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /log/{group}/... ==> Unknown group name");
    	}
    	//System.out.println("receive to insert : " + listDataToInsert.size());
        DBEntry entry = DBEntry.createInterface(GlobalConfiguration.dbConfig);
        entry.connection.setAutoCommit(false);
        try {
	        String query = "INSERT INTO log (`create_date`, `group`, `system`, `clientUuid`, `clientId`, `data`) VALUES (?, ?, ?, ?, ?, ?)";
	        // prepare the request:
	        PreparedStatement ps = entry.connection.prepareStatement(query);//, Statement.RETURN_GENERATED_KEYS);
	
	        Timestamp now = Timestamp.from(Instant.now());
	        int batchId = 0;
			for (MultipleLogElement dataToInsert : listDataToInsert) {
				Timestamp create_date = now;
				if (dataToInsert.time != null) {
					try {
						create_date = Timestamp.from(Instant.parse(dataToInsert.time));
					}  catch (Exception ex) {
						throw new InputException("time", "url: ?time=... ==> is not an iso 8601 time format");
					}
				}
	
		        int iii = 1;
		        if (create_date == null) {
		        	ps.setNull(iii++, Types.INTEGER);
		        } else {
		        	ps.setTimestamp(iii++, create_date);
		        }
		        ps.setLong(iii++, group.id);
		        if (system == null) {
		        	ps.setNull(iii++, Types.VARCHAR);
		        } else {
		        	ps.setString(iii++, system);
		        }
		        if (clientUuid == null) {
		        	ps.setNull(iii++, Types.BIGINT);
		        } else {
		        	ps.setLong(iii++, clientUuid);
		        }
		        if (dataToInsert.id == null) {
		        	ps.setNull(iii++, Types.BIGINT);
		        } else {
		        	ps.setLong(iii++, dataToInsert.id);
		        }
		        if (dataToInsert.data == null) {
		        	ps.setNull(iii++, Types.VARCHAR);
		        } else {
		        	ps.setString(iii++, dataToInsert.data);
		        }
	
	            ps.addBatch();
	            batchId++;
	
	            if (batchId % 1000 == 0 || batchId == listDataToInsert.size()) {
	            	//System.out.println("    insert : " + (batchId%1000) + " elements");
	            	//long start_time2 = System.currentTimeMillis();
	            	ps.executeBatch(); // Execute every 1000 items.
	            	//long end_time2 = System.currentTimeMillis();
	            	//System.out.println("    inject time : " + (end_time2 - start_time2) + " ms");
	            	ps.clearBatch();
	            }
			}		
	        entry.connection.setAutoCommit(true);
	    } finally {
	    	entry.close();
	    	entry = null;
	    }
    	long end_time = System.currentTimeMillis();
    	System.out.println("log process time=" + (end_time - start_time) + " ms insert count=" + listDataToInsert.size());
		return Response.status(201).build();
    }
    
}

