package org.kar.karanage.api;

import org.kar.archidata.SqlWrapper;
import org.kar.archidata.WhereCondition;
import org.kar.karanage.model.StateInstant;
import org.kar.karanage.model.StateHistory;
import org.kar.karanage.model.Group;
import jakarta.ws.rs.core.Response;


import org.kar.archidata.annotation.security.PermitAll;
import org.kar.archidata.exception.FailException;
import org.kar.archidata.exception.InputException;

import jakarta.ws.rs.*;
import jakarta.ws.rs.core.MediaType;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.List;


@Path("/state_history")
@Produces({MediaType.APPLICATION_JSON})
public class StateHistoryResource {
 
    @GET
    @Path("{group}/{topic:.*}")
    //@RolesAllowed("USER")
    @PermitAll
    public static String get(
    		@PathParam("group") String groupName,
    		@PathParam("topic") String topic,
    		@QueryParam("since") String since,
    		@QueryParam("sinceId") Integer sinceId,
    		@QueryParam("limit") Integer limit,
    		@QueryParam("mode") String mode) throws Exception {
    	// Keep Group ID:
    	if (groupName == null || groupName.length() == 0) {
    		throw new InputException("group", "Missing URL parameter /state/{group}/...");
    	}
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /state/{group}/... ==> Unknown group name");
    	}
    	if (topic == null || topic.length() == 0) {
    		throw new InputException("topic", "Missing URL parameter /state/xxx/{topic}/...");
    	}
    	if (limit == null || limit == 0) {
    		limit = 100;
    	}
    	if (limit > 100) {
    		limit = 100;
    	}
    	if (limit < 0) {
    		throw new InputException("limit", "Limit must be inside [1..100]");
    	}
    	if (since != null && sinceId != null) {
    		throw new InputException("since,sinceId", "The 2 parameters can not be set at the same time...");
    	}
    	
    	List<StateHistory> datas = null;
    	if (since != null) {
    		Timestamp time = null;
    		try {
    			time = Timestamp.from(Instant.parse(since));
    		}  catch (Exception ex) {
    			throw new InputException("since", "url: ?since=... ==> is not an iso 8601 time format");
    		}
    		datas = SqlWrapper.getsWhere(StateHistory.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("topic", "=", topic),
    						new WhereCondition("modify_date", ">", time)
    						),
    				"modify_date",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "Topic has no new data or does not exist: '" + topic + "'");
        	}
    	} else if (sinceId != null) {
    		datas = SqlWrapper.getsWhere(StateHistory.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("topic", "=", topic),
    						new WhereCondition("id", ">", sinceId)
    						),
    				"id",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "Topic has no new data or does not exist: '" + topic + "'");
        	}
    	} else {
    		datas = SqlWrapper.getsWhere(StateHistory.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("topic", "=", topic)
    						),
    				"id",
    				true,
    				limit);
        	if (datas == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "Topic does not exist: '" + topic + "'");
        	}
    	}

    	StringBuilder out = new StringBuilder("[");
    	boolean first = true;
    	for (StateInstant data : datas) {
    		if (first) {
    			first = false;
    		} else {
    			out.append(",");
    		}
    		out.append("{ \"id\": " + data.id + ", \"time\": ");
        	if ("raw".equals(mode)) {
        		double seconds = data.modify_date.toInstant().toEpochMilli() * 0.001;
        		out.append(seconds);
    		} else {
	    		out.append("\"");
	    		out.append(data.modify_date.toInstant().toString());
	    		out.append("\"");
    		}
    		out.append(", \"state\":");
    		if (data.state == null) {
    			out.append("null");
    		} else {
    			out.append("\"");
    			out.append(data.state);
    			out.append("\"");
    		}
    		out.append(", \"data\":" + data.data + "}");
    	}
    	out.append("]");
    	return out.toString();
    }
    
}

