package org.kar.karanage.api;

import org.kar.archidata.SqlWrapper;
import org.kar.archidata.WhereCondition;
import org.kar.karanage.model.StateInstant;
import org.kar.karanage.model.StateHistory;
import org.kar.karanage.model.Group;
import jakarta.ws.rs.core.Response;


import org.kar.archidata.annotation.security.PermitAll;
import org.kar.archidata.exception.FailException;
import org.kar.archidata.exception.InputException;

import jakarta.ws.rs.*;
import jakarta.ws.rs.core.MediaType;

import java.sql.Timestamp;
import java.time.Instant;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.util.List;


@Path("/state")
@Produces({MediaType.APPLICATION_JSON})
public class StateResource {
 
    @GET
    @Path("{group}/{topic:.*}")
    //@RolesAllowed("USER")
    @PermitAll
    public static String getWithTopic(
    		@PathParam("group") String groupName,
    		@PathParam("topic") String topic,
    		@QueryParam("since") String since,
    		@QueryParam("mode") String mode) throws Exception {
    	if (groupName == null || groupName.length() == 0) {
    		throw new InputException("group", "Missing URL parameter /state/{group}/...");
    	}
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /state/{group}/... ==> Unknown group name");
    	}
    	StateInstant data = null;
    	if (since != null) {
    		Timestamp time = null;
    		try {
    			time = Timestamp.from(Instant.parse(since));
    		}  catch (Exception ex) {
    			throw new InputException("since", "url: ?since=... ==> is not an iso 8601 time format");
    		}
    		data = SqlWrapper.getWhere(StateInstant.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("topic", "=", topic),
    						new WhereCondition("modify_date", ">", time)
    						),
    				true);
        	if (data == null) {
        	    //throw new FailException(Response.Status.NOT_FOUND, "Topic has no new data or does not exist: '" + topic + "'");
        		return "";
        	}
    	} else {
    		data = SqlWrapper.getWhere(StateInstant.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("topic", "=", topic)
    						),
    				true);
        	if (data == null) {
        	    throw new FailException(Response.Status.NOT_FOUND, "Topic does not exist: '" + topic + "'");
        	}
    	}
    	StringBuilder out = new StringBuilder("{ \"time\": ");
    	if ("raw".equals(mode)) {
    		double seconds = data.modify_date.toInstant().toEpochMilli() * 0.001;
    		out.append(seconds);
    	} else {
    		out.append("\"");
    		out.append(data.modify_date.toInstant().toString());
    		out.append("\"");
    	}
		out.append(", \"state\":");
		if (data.state == null) {
			out.append("null");
		} else {
			out.append("\"");
			out.append(data.state);
			out.append("\"");
		}
		out.append(", \"data\":");
		out.append(data.data);
		out.append("}");
    	return out.toString();
    }

    @GET
    @Path("{group}")
    //@RolesAllowed("USER")
    @PermitAll
    public String get(
    		@PathParam("group") String groupName,
    		@QueryParam("since") String since,
    		@QueryParam("mode") String mode) throws Exception {
    	if (groupName == null || groupName.length() == 0) {
    		throw new InputException("group", "Missing URL parameter /state/{group}/...");
    	}
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /state/{group}/... ==> Unknown group name");
    	}

    	List<StateInstant> datas = null;
    	if (since != null) {
    		Timestamp time = null;
    		try {
    			time = Timestamp.from(Instant.parse(since));
    		}  catch (Exception ex) {
    			throw new InputException("since", "url: ?since=... ==> is not an iso 8601 time format");
    		}
    		datas = SqlWrapper.getsWhere(StateInstant.class,
    				List.of(
    						new WhereCondition("group", "=", group.id),
    						new WhereCondition("modify_date", ">", time)
    						),
    				true);
    	} else {
    		datas = SqlWrapper.getsWhere(StateInstant.class,
    				List.of(
    						new WhereCondition("group", "=", group.id)
    						),
    				true);
    	}
    	StringBuilder out = new StringBuilder("[");
    	boolean first = true;
    	for (StateInstant data : datas) {
    		if (first) {
    			first = false;
    		} else {
    			out.append(",");
    		}
    		out.append("{ \"time\": ");
        	if ("raw".equals(mode)) {
        		double seconds = data.modify_date.toInstant().toEpochMilli() * 0.001;
        		out.append(seconds);
        	} else {
        		out.append("\"");
        		out.append(data.modify_date.toInstant().toString());
        		out.append("\"");
        	}
    		out.append(", \"state\":");
    		if (data.state == null) {
    			out.append("null");
    		} else {
    			out.append("\"");
    			out.append(data.state);
    			out.append("\"");
    		}
        	out.append(", \"topic\": \"" + data.topic + "\", \"data\":" + data.data + "}");
    	}
    	out.append("]");
    	return out.toString();
    }
    /*
    public List<DataInstant> get(@PathParam("group") String groupName) throws Exception {
    	Group group = SqlWrapper.getWhere(Group.class, "name", "=", groupName);
    	if (group == null) {
    		throw new Exception("Missing Group !!!");
    	}
    	return SqlWrapper.getsWhere(DataInstant.class, "group", "=", group.id, true);
    }
    */

    @POST
    @Path("{group}/{topic:.*}")
    //@RolesAllowed("ADMIN")
    @PermitAll
    @Consumes(MediaType.APPLICATION_JSON)
    public Response post(
    		@PathParam("group") String groupName,
    		@PathParam("topic") String topic,
    		@QueryParam("state") String state,
    		String dataToInsert) throws Exception {
    	StateInstant previous = null;
    	
    	Group group = SqlWrapper.getWhere(Group.class,
    			List.of(
    					new WhereCondition("name", "=", groupName)
    					),
    			false);
    	if (group == null) {
    		throw new InputException("group", "url: /state/{group}/... ==> Unknown group name");
    	}
    	if (group != null) {
    		previous = SqlWrapper.getWhere(StateInstant.class, 
    				List.of(
        					new WhereCondition("group", "=", group.id),
        					new WhereCondition("topic", "=", topic)
        					),
    				true);
    	}
    	if (previous == null) {
    		// insert new data
    		StateInstant data = new StateInstant();
    		data.group = group.id;
    		data.topic = topic;
    		data.data = dataToInsert;
    		data.state = state;
    		previous = SqlWrapper.insert(data);
        	StateHistory dataHistory = new StateHistory(previous);
        	dataHistory.year = true;
        	dataHistory.month = true;
        	dataHistory.week = true;
        	dataHistory.day = true;
        	SqlWrapper.insert(dataHistory);
    		return Response.status(201).build();
    	}
    	// update Data
    	try {
	    	previous.data = dataToInsert;
	    	previous.state = state;
	    	SqlWrapper.update(previous, previous.id, List.of("data", "state"));
	    	// add it in history:
	        StateHistory dataHistory = new StateHistory(previous);
	        OffsetDateTime now = OffsetDateTime.now( ZoneOffset.UTC );
	        OffsetDateTime sqlTime = OffsetDateTime.ofInstant(Instant.ofEpochMilli(previous.modify_date.getTime()), ZoneOffset.UTC);
	        
	        now.getDayOfMonth();
	        if (sqlTime.getYear() != now.getYear()) {
	        	dataHistory.year = true;
	        }
	        if (sqlTime.getMonth() != now.getMonth()) {
	        	dataHistory.month = true;
	        }
	        if (!sqlTime.getDayOfWeek().equals(now.getDayOfWeek())) {
	        	dataHistory.week = true;
	        }
	        if (sqlTime.getDayOfYear() != now.getDayOfYear()) {
	        	dataHistory.day = true;
	        }
	        if (sqlTime.getHour() != now.getHour()) {
	        	dataHistory.hour = true;
	        }
	        dataHistory.modify_date = Timestamp.valueOf(now.atZoneSameInstant(ZoneOffset.UTC).toLocalDateTime());
	
	        SqlWrapper.insert(dataHistory);
    	} catch (Exception ex) {
    		System.out.print("get exception: " + ex);
    	}
		return Response.status(204).build();
    }
    
}

